@@ -52,6 +52,50 @@
 
 #include <qplatformdefs.h> //for readlink
 
+namespace
+{
+
+/**
+ * Checks if name ends in one of the supported icon formats (i.e. .png)
+ * and returns the name without the extension if it does.
+ */
+QString removeIconExtension(const QString &name)
+{
+    if (name.endsWith(QLatin1String(".png")) //
+        || name.endsWith(QLatin1String(".xpm")) //
+        || name.endsWith(QLatin1String(".svg"))) {
+        return name.left(name.length() - 4);
+    } else if (name.endsWith(QLatin1String(".svgz"))) {
+        return name.left(name.length() - 5);
+    }
+
+    return name;
+}
+
+QStringList deduplicateIconsByName(const QStringList &icons)
+{
+    // Eliminate duplicate entries (same icon in different directories)
+    QStringList result;
+    QStringList entries;
+    for (const auto &icon : icons) {
+        const int n = icon.lastIndexOf(QLatin1Char('/'));
+        QString name;
+        if (n == -1) {
+            name = icon;
+        } else {
+            name = icon.mid(n + 1);
+        }
+        name = removeIconExtension(name);
+        if (!entries.contains(name)) {
+            entries += name;
+            result += icon;
+        }
+    }
+    return result;
+}
+
+} // namespace
+
 /**
  * Function to convert an uint32_t to AARRGGBB hex values.
  *
@@ -95,6 +139,7 @@ public:
     KIconThemeNode(const KIconThemeNode &) = delete;
     KIconThemeNode &operator=(const KIconThemeNode &) = delete;
 
+    [[nodiscard]] QStringList queryIcons() const;
     void queryIcons(QStringList *lst, int size, KIconLoader::Context context) const;
     void queryIconsByContext(QStringList *lst, int size, KIconLoader::Context context) const;
     QString findIcon(const QString &name, int size, KIconLoader::MatchType match) const;
@@ -112,6 +157,11 @@ KIconThemeNode::~KIconThemeNode()
     delete theme;
 }
 
+QStringList KIconThemeNode::queryIcons() const
+{
+    return theme->queryIcons();
+}
+
 void KIconThemeNode::queryIcons(QStringList *result, int size, KIconLoader::Context context) const
 {
     // add the icons of this theme to it
@@ -221,8 +271,7 @@ void KIconLoaderPrivate::clear()
     /* antlarr: There's no need to delete d->mpThemeRoot as it's already
     deleted when the elements of d->links are deleted */
     qDeleteAll(links);
-    delete[] mpGroups;
-    mpGroups = nullptr;
+    mpGroups.clear();
     mPixmapCache.clear();
     m_appname.clear();
     searchPaths.clear();
@@ -370,7 +419,7 @@ void KIconLoaderPrivate::init(const QString &_appname, const QStringList &extraS
     // load default sizes
     initIconThemes();
     KIconTheme *defaultSizesTheme = links.empty() ? nullptr : links.first()->theme;
-    mpGroups = new KIconGroup[static_cast<int>(KIconLoader::LastGroup)];
+    mpGroups.resize(int(KIconLoader::LastGroup));
     for (KIconLoader::Group i = KIconLoader::FirstGroup; i < KIconLoader::LastGroup; ++i) {
         if (groups[i] == nullptr) {
             break;
@@ -545,19 +594,6 @@ void KIconLoader::drawOverlays(const QStringList &overlays, QPixmap &pixmap, KIc
     d->drawOverlays(this, group, state, pixmap, overlays);
 }
 
-QString KIconLoaderPrivate::removeIconExtension(const QString &name) const
-{
-    if (name.endsWith(QLatin1String(".png")) //
-        || name.endsWith(QLatin1String(".xpm")) //
-        || name.endsWith(QLatin1String(".svg"))) {
-        return name.left(name.length() - 4);
-    } else if (name.endsWith(QLatin1String(".svgz"))) {
-        return name.left(name.length() - 5);
-    }
-
-    return name;
-}
-
 void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, QSize &size, int &state) const
 {
     if ((state < 0) || (state >= KIconLoader::LastState)) {
@@ -775,7 +811,7 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size, qrea
             // The next block removes the last word including the -, "a-b-symbolic" will become "a-b"
             // We remove it beforehand, "a-b-symbolic" now is "a-symbolic" and we'll add it back later.
             if (isSymbolic) {
-                currentName.remove(QLatin1String("-symbolic"));
+                currentName.chop(strlen("-symbolic"));
 
                 // Handle cases where the icon lacks a symbolic version.
                 // For example, "knotes-symbolic" doesn't exist and has no fallback or generic version.
@@ -910,7 +946,7 @@ QString KIconLoader::iconPath(const QString &_name, int group_or_size, bool canR
         return _name;
     }
 
-    QString name = d->removeIconExtension(_name);
+    QString name = removeIconExtension(_name);
 
     QString path;
     if (group_or_size == KIconLoader::User) {
@@ -1059,7 +1095,7 @@ QPixmap KIconLoader::loadScaledIcon(const QString &_name,
     // we need to honor resource :/ paths and QDir::searchPaths => use QDir::isAbsolutePath, see bug 434451
     const bool absolutePath = QDir::isAbsolutePath(name);
     if (!absolutePath) {
-        name = d->removeIconExtension(name);
+        name = removeIconExtension(name);
     }
 
     // Don't bother looking for an icon with no name.
@@ -1197,7 +1233,7 @@ QMovie *KIconLoader::loadMovie(const QString &name, KIconLoader::Group group, in
 #if KICONTHEMES_BUILD_DEPRECATED_SINCE(6, 5)
 QString KIconLoader::moviePath(const QString &name, KIconLoader::Group group, int size) const
 {
-    if (!d->mpGroups) {
+    if (d->mpGroups.empty()) {
         return QString();
     }
 
@@ -1247,7 +1283,7 @@ QStringList KIconLoader::loadAnimated(const QString &name, KIconLoader::Group gr
 {
     QStringList lst;
 
-    if (!d->mpGroups) {
+    if (d->mpGroups.empty()) {
         return lst;
     }
 
@@ -1305,7 +1341,7 @@ KIconTheme *KIconLoader::theme() const
 
 int KIconLoader::currentSize(KIconLoader::Group group) const
 {
-    if (!d->mpGroups) {
+    if (d->mpGroups.empty()) {
         return -1;
     }
 
@@ -1346,24 +1382,19 @@ QStringList KIconLoader::queryIconsByContext(int group_or_size, KIconLoader::Con
         themeNode->queryIconsByContext(&result, size, context);
     }
 
-    // Eliminate duplicate entries (same icon in different directories)
-    QString name;
-    QStringList res2;
-    QStringList entries;
-    for (const auto &icon : std::as_const(result)) {
-        const int n = icon.lastIndexOf(QLatin1Char('/'));
-        if (n == -1) {
-            name = icon;
-        } else {
-            name = icon.mid(n + 1);
-        }
-        name = d->removeIconExtension(name);
-        if (!entries.contains(name)) {
-            entries += name;
-            res2 += icon;
-        }
+    return deduplicateIconsByName(result);
+}
+
+QStringList KIconLoader::queryIcons() const
+{
+    d->initIconThemes();
+
+    QStringList result;
+    for (const auto &themeNode : std::as_const(d->links)) {
+        result.append(themeNode->queryIcons());
     }
-    return res2;
+
+    return deduplicateIconsByName(result);
 }
 
 QStringList KIconLoader::queryIcons(int group_or_size, KIconLoader::Context context) const
@@ -1386,24 +1417,7 @@ QStringList KIconLoader::queryIcons(int group_or_size, KIconLoader::Context cont
         themeNode->queryIcons(&result, size, context);
     }
 
-    // Eliminate duplicate entries (same icon in different directories)
-    QString name;
-    QStringList res2;
-    QStringList entries;
-    for (const auto &icon : std::as_const(result)) {
-        const int n = icon.lastIndexOf(QLatin1Char('/'));
-        if (n == -1) {
-            name = icon;
-        } else {
-            name = icon.mid(n + 1);
-        }
-        name = d->removeIconExtension(name);
-        if (!entries.contains(name)) {
-            entries += name;
-            res2 += icon;
-        }
-    }
-    return res2;
+    return deduplicateIconsByName(result);
 }
 
 // used by KIconDialog to find out which contexts to offer in a combobox
@@ -1469,12 +1483,10 @@ bool KIconLoader::hasCustomPalette() const
     return d->mCustomColors;
 }
 
-/*** the global icon loader ***/
-Q_GLOBAL_STATIC(KIconLoader, globalIconLoader)
-
 KIconLoader *KIconLoader::global()
 {
-    return globalIconLoader();
+    thread_local KIconLoader loader;
+    return &loader;
 }
 
 void KIconLoader::newIconLoader()
